/**********************************************************************
* $Id$		Lcd_Demo.c				2011-06-02
*//**
* @file		Lcd_Demo.c
* @brief	This example describes how to use LCD on Hitex board
* 			and LogicPD 4.3
* @version	1.0
* @date		02. June. 2011
* @author	NXP MCU SW Application Team
*
* Copyright(C) 2011, NXP Semiconductor
* All rights reserved.
*
***********************************************************************
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* products. This software is supplied "AS IS" without any warranties.
* NXP Semiconductors assumes no responsibility or liability for the
* use of the software, conveys no license or title under any patent,
* copyright, or mask work right to the product. NXP Semiconductors
* reserves the right to make changes in the software without
* notification. NXP Semiconductors also make no representation or
* warranty that such application will be suitable for the specified
* use without further testing or modification.
* Permission to use, copy, modify, and distribute this software and its
* documentation is hereby granted, under NXP Semiconductors'
* relevant copyright in the software, without fee, provided that it
* is used in conjunction with NXP Semiconductors microcontrollers.  This
* copyright, permission, and disclaimer notice must appear in all copies of
* this code.
**********************************************************************/
#include "lpc43xx_gpio.h"
#include "lpc43xx_scu.h"
//#include "lpc43xx_libcfg.h"
#include "lpc43xx_cgu.h"
#include "lpc4357_uart.h"
#include "LCD.h"
#include "AsciiLib.h"



#define C_GLCD_CLK_PER_LINE     (C_GLCD_H_SIZE + C_GLCD_H_PULSE + C_GLCD_H_FRONT_PORCH + C_GLCD_H_BACK_PORCH)
#define C_GLCD_LINES_PER_FRAME  (C_GLCD_V_SIZE + C_GLCD_V_PULSE + C_GLCD_V_FRONT_PORCH + C_GLCD_V_BACK_PORCH)
#define C_GLCD_PIX_CLK          (C_GLCD_CLK_PER_LINE * C_GLCD_LINES_PER_FRAME * C_GLCD_REFRESH_FREQ)

uint32_t *framebuffer = (uint32_t *)FRAMEBUFFER_ADDR;
#define LCD_VRAM_BASE_ADDR 	((uint32_t)FRAMEBUFFER_ADDR)

/*-------------------------PRIVATE FUNCTIONS------------------------------*/
/*********************************************************************//**
 * @brief		put a pixel
 * @param[in]	x,y postion of pixel
 * @param[in]	val color value of pixel in 32bit
 * @return 		None
 **********************************************************************/

void putpixel(uint32_t x, uint32_t y, uint32_t val)
{	
 #ifdef MODE_565 
	
	if(x % 2)
	{
			framebuffer[x/2 + y*(C_GLCD_H_SIZE/2)] &= ~(0xffff);
			framebuffer[x/2 + y*(C_GLCD_H_SIZE/2)] |= val;
	}
	else
	{
			framebuffer[x/2 + y*(C_GLCD_H_SIZE/2)] &= ~(0xffff<< 16);
			framebuffer[x/2 + y*(C_GLCD_H_SIZE/2)] |= val << 16;
	}
	
#elif defined MODE_888	
		framebuffer[x+y*C_GLCD_H_SIZE] = val;
	
#endif
}

/*-------------------------MAIN FUNCTION------------------------------*/
/*********************************************************************//**
 * @brief		Main program body
 * @param[in]	None
 * @return 		int
 **********************************************************************/
uint32_t tempx, tempy;
extern const unsigned char image[];

void LCD_Init (void) {                       /* Main Program                       */

	uint32_t i,j;
	
	scu_pinmux(0x04, 7, MD_PUP, FUNC0);		/* LCD_DCLK @ P4.7 */
	scu_pinmux(0x04, 6, MD_PUP, FUNC2); 	/* LCD_ENAB_M @ P4.6 */
	scu_pinmux(0x04, 5, MD_PUP, FUNC2);		/* LCD_FP @ P4.5 */	
	scu_pinmux(0x07, 0, MD_PUP, FUNC0);		/* LCD_LE @ P7.0 */	
	scu_pinmux(0x07, 6, MD_PUP, FUNC3);		/* LCD_LP @ P7.6 */
	
	scu_pinmux(0x07, 7, MD_PUP, FUNC0); 	/* LCD_PWR @ P7.7 */		
	
	scu_pinmux(0x04, 1, MD_PUP, FUNC2);		/* LCD_VD_0 @ P4.1 */	
	scu_pinmux(0x04, 4, MD_PUP, FUNC2);		/* LCD_VD_1 @ P4.4 */	
	scu_pinmux(0x04, 3, MD_PUP, FUNC2);		/* LCD_VD_2 @ P4.3 */	
	scu_pinmux(0x04, 2, MD_PUP, FUNC2);		/* LCD_VD_3 @ P4.2 */		
	scu_pinmux(0x08, 7, MD_PUP, FUNC3);		/* LCD_VD_4 @ P8.7 */	
	scu_pinmux(0x08, 6, MD_PUP, FUNC3);		/* LCD_VD_5 @ P8.6 */	
	scu_pinmux(0x08, 5, MD_PUP, FUNC3);		/* LCD_VD_6 @ P8.5 */	
	scu_pinmux(0x08, 4, MD_PUP, FUNC3);		/* LCD_VD_7 @ P8.4 */
	scu_pinmux(0x07, 5, MD_PUP, FUNC3);		/* LCD_VD_8 @ P8.4 */	
	scu_pinmux(0x04, 8, MD_PUP, FUNC2);		/* LCD_VD_9 @ P8.4 */
	scu_pinmux(0x04, 10, MD_PUP, FUNC2);	/* LCD_VD_10 @ P4.10 */	
	scu_pinmux(0x04, 9, MD_PUP, FUNC2); 	/* LCD_VD_11 @ P4.9 */	
	scu_pinmux(0x08, 3, MD_PUP, FUNC3); 	/* LCD_VD_12 @ P8.3 */	
	scu_pinmux(0x0B, 6, MD_PUP, FUNC2); 	/* LCD_VD_13 @ PB.6 */	
	scu_pinmux(0x0B, 5, MD_PUP, FUNC2); 	/* LCD_VD_14 @ PB.5 */	 
	scu_pinmux(0x0B, 4, MD_PUP, FUNC2); 	/* LCD_VD_15 @ PB.4 */	
	scu_pinmux(0x07, 4, MD_PUP, FUNC3); 	/* LCD_VD_16 @ PB.4 */
	scu_pinmux(0x07, 3, MD_PUP, FUNC3); 	/* LCD_VD_17 @ PB.4 */	
	scu_pinmux(0x07, 2, MD_PUP, FUNC3); 	/* LCD_VD_18 @ P7.2 */	
	scu_pinmux(0x07, 1, MD_PUP, FUNC3); 	/* LCD_VD_19 @ P7.1 */	
	scu_pinmux(0x0B, 3, MD_PUP, FUNC2); 	/* LCD_VD_20 @ PB.3 */	
	scu_pinmux(0x0B, 2, MD_PUP, FUNC2); 	/* LCD_VD_21 @ PB.2 */	
	scu_pinmux(0x0B, 1, MD_PUP, FUNC2); 	/* LCD_VD_22 @ PB.1 */	
	scu_pinmux(0x0B, 0, MD_PUP, FUNC2); 	/* LCD_VD_23 @ PB.0 */

	GPIO_SetDir(3,1<<8, 1);	/*LCDLE*/
	GPIO_SetValue(3, 1<<8);

	GPIO_SetDir(3,1<<15, 1);	/*LCDLE*/
	GPIO_SetValue(3, 1<<15);
	
	
	LPC_RGU->RESET_CTRL0 = (1UL << 16);

  /* Configure LCD controller ------------------------------------------------*/
  LPC_RGU->RESET_CTRL0 = (1U << 16);
  while ((LPC_RGU->RESET_ACTIVE_STATUS0 & (1U << 16)) == 0);

  LPC_LCD->CTRL &= ~(1 << 0);           /* Disable LCD                        */

  LPC_LCD->INTMSK = 0;                  /* Disable all interrupts             */

  LPC_LCD->UPBASE = FRAMEBUFFER_ADDR & ~7UL ;
  LPC_LCD->LPBASE = FRAMEBUFFER_ADDR & ~7UL ;
						 
									 
	// init Horizontal Timing
	LPC_LCD->TIMH = 0; //reset TIMH before set value
	LPC_LCD->TIMH |= (C_GLCD_H_BACK_PORCH - 1)<<24;
	LPC_LCD->TIMH |= (C_GLCD_H_FRONT_PORCH - 1)<<16;
	LPC_LCD->TIMH |= (C_GLCD_H_PULSE - 1)<<8;
	LPC_LCD->TIMH |= ((C_GLCD_H_SIZE/16) - 1)<<2;
	
	// init Vertical Timing
	LPC_LCD->TIMV = 0;  //reset TIMV value before setting
	LPC_LCD->TIMV |= (C_GLCD_V_BACK_PORCH)<<24;
	LPC_LCD->TIMV |= (C_GLCD_V_FRONT_PORCH)<<16;
	LPC_LCD->TIMV |= (C_GLCD_V_PULSE - 1)<<10;
	LPC_LCD->TIMV |=  C_GLCD_V_SIZE- 1;									 

	// bypass inrenal clk divider
	LPC_LCD->POL |=(1<<26);
	// clock source for the LCD block is HCLK
	LPC_LCD->POL &= ~(1<<5);
	// LCDFP pin is active LOW and inactive HIGH
	LPC_LCD->POL |= (1<<11);
	// LCDLP pin is active LOW and inactive HIGH
	LPC_LCD->POL |= (1<<12);
	// data is driven out into the LCD on the falling edge
	LPC_LCD->POL &= ~(1<<13);
	// active high
	LPC_LCD->POL &= ~(1<<14);
	
	
	LPC_LCD->POL &= ~(0x3FF <<16);
	LPC_LCD->POL |= (C_GLCD_H_SIZE-1)<<16;								
  LPC_LCD->LE    = (1    << 16) |       /* LCDLE Enabled: 1, Disabled: 0      */
                   (9    <<  0) ;       /* Line-end delay: LCDCLK clocks - 1  */
									
									 
#ifdef MODE_565 

  LPC_LCD->CTRL  = (1    << 11) |       /* LCD Power Enable                   */
                   (1    <<  5) |       /* 0 = STN display, 1: TFT display    */
                   (6    <<  1) ;       /* Bits per pixel: 565             */
									 
#elif defined MODE_888	 

  LPC_LCD->CTRL  = (1    << 11) |       /* LCD Power Enable                   */
                   (1    <<  5) |       /* 0 = STN display, 1: TFT display    */
                   (5    <<  1) ;       /* Bits per pixel: 24              */
#endif			


  for (i = 0; i < 256; i++) {
    LPC_LCD->PAL[i] = 0;                /* Clear palette                      */
  }
  LPC_LCD->CTRL |= (1 <<  0);           /* LCD enable                         */

  /* Turn on backlight -------------------------------------------------------*/
  LPC_GPIO_PORT->DIR[3] |= (1 << 8);
  LPC_GPIO_PORT->SET[3] =  (1 << 8);
	
	
	
	#ifdef MODE_565 
	/*Fill Colorbar only*/
		for(i=0;i<C_GLCD_H_SIZE*C_GLCD_V_SIZE/8;i++){
		framebuffer[i] = Red | (Red << 16) ;
		}
		for(i=C_GLCD_H_SIZE*C_GLCD_V_SIZE/8;i<C_GLCD_H_SIZE*C_GLCD_V_SIZE*2/4;i++){
		framebuffer[i] = Green | (Green << 16);
		}
		for(i=C_GLCD_H_SIZE*C_GLCD_V_SIZE*2/8;i<C_GLCD_H_SIZE*C_GLCD_V_SIZE*3/4;i++){
		framebuffer[i] = Blue | (Blue << 16);
		}
		
		for(i=C_GLCD_H_SIZE*C_GLCD_V_SIZE*3/8;i<C_GLCD_H_SIZE*C_GLCD_V_SIZE;i++){
		framebuffer[i] = Yellow | (Yellow << 16);
		}
		
	/* Fill NXP logo */
	for(j=0;j<LOGO_HEIGHT;j++){
		for(i=0;i<LOGO_WIDTH;i++){
			putpixel(i,j, (image[(i+j*LOGO_WIDTH)+LOGO_WIDTH*LOGO_HEIGHT*2] & 0xf8)<<8
				| (image[(i+j*LOGO_WIDTH)+LOGO_WIDTH*LOGO_HEIGHT] & 0xfc)<<3 | (image[(i+j*LOGO_WIDTH)] & 0xf8)>>3 );
		}
	}
									 
#elif defined MODE_888	 

	/*Fill Colorbar only*/
		for(i=0;i<C_GLCD_H_SIZE*C_GLCD_V_SIZE/4;i++){
		framebuffer[i] = Red;
		}
		for(i=C_GLCD_H_SIZE*C_GLCD_V_SIZE/4;i<C_GLCD_H_SIZE*C_GLCD_V_SIZE*2/4;i++){
		framebuffer[i] = Green;
		}
		for(i=C_GLCD_H_SIZE*C_GLCD_V_SIZE*2/4;i<C_GLCD_H_SIZE*C_GLCD_V_SIZE*3/4;i++){
		framebuffer[i] = Blue;
		}
		
		for(i=C_GLCD_H_SIZE*C_GLCD_V_SIZE*3/4;i<C_GLCD_H_SIZE*C_GLCD_V_SIZE;i++){
		framebuffer[i] = Yellow;
		}
		
	/* Fill NXP logo */
	for(j=0;j<LOGO_HEIGHT;j++){
		for(i=0;i<LOGO_WIDTH;i++){
			putpixel(i,j, image[(i+j*LOGO_WIDTH)+LOGO_WIDTH*LOGO_HEIGHT*2]<<16
				| image[(i+j*LOGO_WIDTH)+LOGO_WIDTH*LOGO_HEIGHT]<<8 | image[(i+j*LOGO_WIDTH)]);
		}
		}
#endif		

}

void delay_ms(uint32_t ms)    
{ 
	uint16_t i,j; 
	for( i = 0; i < ms; i++ )
	{ 
		for( j = 0; j < 110; j++ );
	}
} 
/*************************************************************************
 * Function Name: lcd_Clear
 * Parameters: uint32_t color
 *
 * Return: none
 *
 * Description: LCD screen clearing
 *
 *************************************************************************/
void lcd_Clear(uint32_t color)
{
	unsigned long i;
	unsigned long value;
	unsigned long *pDst = (unsigned long *)LCD_VRAM_BASE_ADDR;
  
    for( i = 0; (C_GLCD_H_SIZE * C_GLCD_V_SIZE) > i; i++)
    {
		#ifdef MODE_565 
			if(i % 2)
			{
					*pDst &= ~(0xffff<< 16);
					*pDst++ |= (color<< 16); 
			}
			else
			{			
					*pDst &= ~(0xffff);
					*pDst |= color; 	
			}
			
		#elif defined MODE_888				
					*pDst++ = color;
		#endif 
    }
		
		
		
}

/******************************************************************************
* Function Name  : LCD_SetPoint
* Description    : 
* Input          : - Xpos: Row Coordinate
*                  - Ypos: Line Coordinate 
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_SetPoint(uint16_t Xpos,uint16_t Ypos,uint32_t color)
{
	unsigned long *pDst = (unsigned long *)LCD_VRAM_BASE_ADDR;
    unsigned long value;
	if( Xpos >= C_GLCD_H_SIZE || Ypos >= C_GLCD_V_SIZE )
	{
		return;
	}

	
#ifdef MODE_565 
	if(Xpos % 2)
	{
			value=Ypos*C_GLCD_H_SIZE/2+Xpos/2;
			pDst = pDst +value ;
			*pDst &= ~(0xffff<< 16);
			*pDst |= (color<< 16); 
	}
	else
	{
			value=Ypos*C_GLCD_H_SIZE/2+Xpos/2;
			pDst = pDst +value ;
			*pDst &= ~(0xffff);
			*pDst |= color; 	
	}
	
#elif defined MODE_888	
	
			value=Ypos*C_GLCD_H_SIZE+Xpos;
			pDst = pDst +value ;
			*pDst = color; 
#endif		
	
}


/******************************************************************************
* Function Name  : LCD_DrawLine
* Description    : Bresenham's line algorithm
* Input          : - x0:
*                  - y0:
*       				   - x1:
*       		       - y1:
*                  - color:
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/	 
void LCD_DrawLine(  uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2 , uint32_t color )
{
unsigned int i1,j1,k1,l1; 
int n1=0; 
unsigned char temp=0; 
if(y2<y1)
    {i1=x1;
    l1=y1;
    x1=x2;
    y1=y2;
    x2=i1;
    y2=l1;}
k1=y2-y1; 
if (k1==0) 
    {if (x1>x2) 
        {i1=x1;
        x1=x2;
        x2=i1;} 
     for ( i1=x1;i1<=x2;i1++ ) 
        LCD_SetPoint(i1,y1,color); 
     } 
else
     {if(x2>=x1)
         {temp=1;
         j1=x2-x1;}
      else 
         j1=x1-x2;
      i1=x1;
      l1=y1;
      LCD_SetPoint(i1,l1,color);
      if(temp&&(k1<=j1))
          while(i1!=x2)
              {if(n1<0)
                  {i1=i1+1;
                  n1=n1+(y2-l1);}
               else 
                  {i1=i1+1;
                  l1=l1+1;
                  n1=n1+(y2-l1)-(x2-i1);} 
               LCD_SetPoint(i1,l1,color);}
      else if(temp&&(k1>j1))
          while(l1!=y2)
              {j1=x2-i1;
              k1=y2-l1; 
              if(n1<0)
                  {i1=i1+1;
                  l1=l1+1;
                  n1=n1+k1-j1;} 
              else 
                  {l1=l1+1;
                  n1=n1-j1;}
              LCD_SetPoint(i1,l1,color);} 
      else if(!temp&&(k1<=j1))
          while(i1!=x2) 
              {j1=i1-x2;
              k1=y2-l1; 
              if(n1<0)
                  {i1=i1-1;
                  n1=n1+k1;} 
              else 
                  {i1=i1-1;
                  l1=l1+1;
                  n1=n1+k1-j1;}
              LCD_SetPoint(i1,l1,color);}
      else if(!temp &&(k1>j1))
          while(l1!=y2) 
              {j1=i1-x2;
              k1=y2-l1; 
              if(n1<0)
                  {i1=i1-1;
                  l1=l1+1;
                  n1=n1+k1-j1;} 
              else 
                  {l1=l1+1;
                  n1=n1-j1;}
                  LCD_SetPoint(i1,l1,color);}
               }
} 

/******************************************************************************
* Function Name  : LCD_Drawcircle
* Description    : Draw round
* Input  
* int x,         : - x0:
* int y,         : - y0:
* int r,      	 : - radius:
* int color      : - color:
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/	 


void LCD_Drawcircle(int x0, int y0, int r, int color)
{
   int  draw_x0, draw_y0;                        // ͼ
    int  draw_x1, draw_y1;        
    int  draw_x2, draw_y2;        
    int  draw_x3, draw_y3;        
    int  draw_x4, draw_y4;        
    int  draw_x5, draw_y5;        
    int  draw_x6, draw_y6;        
    int  draw_x7, draw_y7;        
    int  xx, yy;                                        // ԲƱ

    int  di;                                                // ߱
   
   /*  */
   if(0==r) return;
   
   /* 8(04590135180225270)ʾ */
   draw_x0 = draw_x1 = x0;
   draw_y0 = draw_y1 = y0 + r;
   if(draw_y0<C_GLCD_V_SIZE) LCD_SetPoint(draw_x0, draw_y0, color);        // 90
        
   draw_x2 = draw_x3 = x0;
   draw_y2 = draw_y3 = y0 - r;
   if(draw_y2>=0) LCD_SetPoint(draw_x2, draw_y2, color);                        // 270
   
        
   draw_x4 = draw_x6 = x0 + r;
   draw_y4 = draw_y6 = y0;
   if(draw_x4<C_GLCD_H_SIZE) LCD_SetPoint(draw_x4, draw_y4, color);        // 0
   
   draw_x5 = draw_x7 = x0 - r;
   draw_y5 = draw_y7 = y0;
   if(draw_x5>=0) LCD_SetPoint(draw_x5, draw_y5, color);                        // 180   
   if(1==r) return;                                        // 뾶Ϊ1Բ
   
   
   /* ʹBresenhamлԲ */
   di = 3 - 2*r;                                        // ʼ߱
   
   xx = 0;
   yy = r;        
   while(xx<yy)
   {  if(di<0)
          {  di += 4*xx + 6;              
          }
          else
          {  di += 4*(xx - yy) + 10;
          
             yy--;          
                 draw_y0--;
                 draw_y1--;
                 draw_y2++;
                 draw_y3++;
                 draw_x4--;
                 draw_x5++;
                 draw_x6--;
                 draw_x7++;                 
          }
          
          xx++;   
          draw_x0++;
          draw_x1--;
          draw_x2++;
          draw_x3--;
          draw_y4++;
          draw_y5++;
          draw_y6--;
          draw_y7--;
                
        
          /* ҪжϵǰǷЧΧ */
          if( (draw_x0<=C_GLCD_H_SIZE)&&(draw_y0>=0) )        
          {  LCD_SetPoint(draw_x0, draw_y0, color);
          }            
          if( (draw_x1>=0)&&(draw_y1>=0) )        
          {  LCD_SetPoint(draw_x1, draw_y1, color);
          }
          if( (draw_x2<=C_GLCD_H_SIZE)&&(draw_y2<=C_GLCD_V_SIZE) )        
          {  LCD_SetPoint(draw_x2, draw_y2, color);   
          }
          if( (draw_x3>=0)&&(draw_y3<=C_GLCD_V_SIZE) )        
          {  LCD_SetPoint(draw_x3, draw_y3, color);
          }
          if( (draw_x4<=C_GLCD_H_SIZE)&&(draw_y4>=0) )        
          {  LCD_SetPoint(draw_x4, draw_y4, color);
          }
          if( (draw_x5>=0)&&(draw_y5>=0) )        
          {  LCD_SetPoint(draw_x5, draw_y5, color);
          }
          if( (draw_x6<=C_GLCD_H_SIZE)&&(draw_y6<=C_GLCD_V_SIZE) )        
          {  LCD_SetPoint(draw_x6, draw_y6, color);
          }
          if( (draw_x7>=0)&&(draw_y7<=C_GLCD_V_SIZE) )        
          {  LCD_SetPoint(draw_x7, draw_y7, color);
          }
   }
}

/******************************************************************************
* Function Name  : PutChar
* Description    : 
* Input          : - Xpos:
*                  - Ypos:
*				           - ASCI:
*				           - charColor:
*				           - bkColor:
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void PutChar( uint16_t Xpos, uint16_t Ypos, uint8_t ASCI, uint32_t charColor, uint32_t bkColor )
{
	uint32_t i, j;
    uint8_t buffer[16], tmp_char;
    GetASCIICode(buffer,ASCI);
    for( i=0; i<16; i++ )
    {
        tmp_char = buffer[i];
        for( j=0; j<8; j++ )
        {
            if( (tmp_char >> 7 - j) & 0x01 == 0x01 )
            {
                LCD_SetPoint( Xpos + j, Ypos + i, charColor );
            }
            else
            {
                LCD_SetPoint( Xpos + j, Ypos + i, bkColor );
            }
        }
    }
}

/******************************************************************************
* Function Name  : GUI_Text
* Description    : 
* Input          : - Xpos: 
*                  - Ypos: 
*				           - str:
*				           - charColor:
*				           - bkColor:
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void GUI_Text(uint16_t Xpos, uint16_t Ypos, uint8_t *str,uint32_t Color, uint32_t bkColor)
{
    uint8_t TempChar;
    do
    {
        TempChar = *str++;  
        PutChar( Xpos, Ypos, TempChar, Color, bkColor );    
        if( Xpos < C_GLCD_H_SIZE - 8 )
        {
            Xpos += 8;
        } 
        else if ( Ypos < C_GLCD_V_SIZE - 16 )
        {
            Xpos = 0;
            Ypos += 16;
        }   
        else
        {
            Xpos = 0;
            Ypos = 0;
        }    
    }
    while ( *str != 0 );
}

/******************************************************************************
* Function Name  : lcd_display_coord
* Description    : 
* Input          : - Xpos: 
*                  - Ypos: 
* uint16_t color     color
* uint16_t color1    color
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/

void lcd_display_coord(uint16_t x,uint16_t y,uint32_t color,uint32_t color1)
{
    uint16_t dx,dy;
    uint16_t dx1,dy1;
    uint16_t i;
    dx=x;dy=y;
    dx1=x;dy1=y;
    for(i=0;i<40;i++)
    {
     LCD_SetPoint(dx-20,dy,color);
     dx++;
     LCD_SetPoint(dx1,dy1-20,color);
     dy1++;  
    }
    dx=x;dy=y;
    dx1=x;dy1=y;
    for(i=0;i<10;i++)
    {
     LCD_SetPoint(dx-20,dy-20,color1);
     LCD_SetPoint(dx-20,dy+20,color1); 
     dx++; 
     LCD_SetPoint(dx1-20,dy1-20,color1);
     LCD_SetPoint(dx1+20,dy1-20,color1);
     dy1++; 
    }
    dx=x;dy=y;
    dx1=x;dy1=y;
    for(i=0;i<10;i++)
    {
     LCD_SetPoint(dx+10,dy-20,color1);
     LCD_SetPoint(dx+10,dy+20,color1);
      
     dx++; 
     LCD_SetPoint(dx1-20,dy1+10,color1);
     LCD_SetPoint(dx1+20,dy1+10,color1);
     dy1++; 
    }

}
