#ifndef __LPC_4330_IO_H
#define __LPC_4330_IO_H

#include "lpc43xx.h"                    /* lpc43xx definitions                */
#include "lpc_types.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_gpio.h"
#include "lpc4357_timer.h"


/* Example group ----------------------------------------------------------- */
/** @defgroup 
 * @ingroup GPIO_Examples
 * @{
 */
 
/************************** LED DEFINITIONS *************************/
//LED1 P9_4   GPIO5[17]  FUNC4
//LED2 P9_3   GPIO4[15]  FUNC0
//LED3 PC_11  GPIO6[10]  FUNC4
//LED4 P4_0   GPIO2[0]  FUNC0
#define LED1                  1
#define LED2                  2
#define LED3                  4
#define LED4                  8

#define LED1_Pin_name_PORT    9
#define LED1_Pin_name_BIT     4
#define LED1_pin_Function    FUNC4
#define LED1_GPIO_PORT        5     
#define LED1_GPIO_BIT         17   

#define LED2_Pin_name_PORT    9
#define LED2_Pin_name_BIT     3
#define LED2_pin_Function    FUNC0
#define LED2_GPIO_PORT        4     
#define LED2_GPIO_BIT         15   

#define LED3_Pin_name_PORT    0X0C
#define LED3_Pin_name_BIT     11
#define LED3_pin_Function    FUNC4
#define LED3_GPIO_PORT        6     
#define LED3_GPIO_BIT         10   

#define LED4_Pin_name_PORT    7
#define LED4_Pin_name_BIT     7
#define LED4_pin_Function    FUNC0
#define LED4_GPIO_PORT        3    
#define LED4_GPIO_BIT         15


/************************** JOYSTICK DEFINITIONS *************************
*JOYSTICK -> KEY1 :  PC_9 -> GPIO6[8]  FUNC4
*JOYSTICK -> KEY2 :  PD_1 -> GPIO6[15]  FUNC4
*JOYSTICK -> A :     PD_10 -> GPIO6[24]  FUNC4
*JOYSTICK -> B :     PD_11 -> GPIO6[25]  FUNC4
*JOYSTICK -> C :     PD_12 -> GPIO6[26]  FUNC4
*JOYSTICK -> D :     PD_13 -> GPIO6[27]   FUNC4
*JOYSTICK -> PRESS : PD_14 -> GPIO6[28]   FUNC4
 
******************************************************************************/
 

#define KEY1_Pin_name_PORT         0x0c
#define KEY1_Pin_name_BIT          9
#define KEY1_pin_Function          FUNC4
#define KEY1_GPIO_PORT             6     
#define KEY1_GPIO_BIT              8 

#define KEY2_Pin_name_PORT         0x0d
#define KEY2_Pin_name_BIT          1
#define KEY2_pin_Function          FUNC4
#define KEY2_GPIO_PORT             6   
#define KEY2_GPIO_BIT              15 


#define JOY_A_Pin_name_PORT        0x0d
#define JOY_A_Pin_name_BIT         10
#define JOY_A_pin_Function         FUNC4
#define JOY_A_GPIO_PORT            6     
#define JOY_A_GPIO_BIT             24  

#define JOY_B_Pin_name_PORT        0x0d
#define JOY_B_Pin_name_BIT         11
#define JOY_B_pin_Function         FUNC4
#define JOY_B_GPIO_PORT            6     
#define JOY_B_GPIO_BIT             25  
 
#define JOY_C_Pin_name_PORT        0x0d
#define JOY_C_Pin_name_BIT         12
#define JOY_C_pin_Function         FUNC4
#define JOY_C_GPIO_PORT            6     
#define JOY_C_GPIO_BIT             26  


#define JOY_D_Pin_name_PORT        0x0d
#define JOY_D_Pin_name_BIT         13
#define JOY_D_pin_Function         FUNC4
#define JOY_D_GPIO_PORT            6     
#define JOY_D_GPIO_BIT             27  

#define JOY_PRESS_Pin_name_PORT    0x0d
#define JOY_PRESS_Pin_name_BIT     14
#define JOY_PRESS_pin_Function     FUNC4
#define JOY_PRESS_GPIO_PORT        6     
#define JOY_PRESS_GPIO_BIT         28 

/*******************************LCD DEFINITIONS***************************/
/*
*WR   ->P1_0 ( GPIO0[4] )    FUNC0
*CS   ->P2_2 ( GPIO5[2] )    FUNC4
*RD   ->P1_7 ( GPIO1[0] )    FUNC0
*RS   ->P1_8 ( GPIO1[1] )    FUNC0
*D0-D15  P4_0...6  P5_0...7  p6_12  ( GPIO2[0...15] )
*/



/*********************************SSPI0 DEFININS **************************************/

#define SSP0_MISO_Pin_name_PORT    0X0F
#define SSP0_MISO_Pin_name_BIT     2
#define SSP0_MISO_Pin_Function     FUNC2

#define SSP0_MOSI_Pin_name_PORT    0X0F
#define SSP0_MOSI_Pin_name_BIT     3
#define SSP0_MOSI_Pin_Function     FUNC2

#define SSP0_SCK_Pin_name_PORT     0x0F
#define SSP0_SCK_Pin_name_BIT      0
#define SSP0_SCK_Pin_Function      FUNC0

#define SSP0_CS_Pin_name_PORT      0X0F
#define SSP0_CS_Pin_name_BIT       1
#define SSP0_CS_Pin_Function       FUNC4
#define SSP0_CS_GPIO_PORT          7     
#define SSP0_CS_GPIO_BIT           16

/******************************* 4.3 inch LCD TOUCHPANEL *****************************************/
#define Touch_IRQ_Pin_name_PORT      0X0A
#define Touch_IRQ_Pin_name_BIT       3
#define Touch_IRQ_Pin_Function       FUNC0
#define Touch_IRQ_GPIO_PORT          4     
#define Touch_IRQ_GPIO_BIT           10


#define Touch_BUSY_Pin_name_PORT      0X0A
#define Touch_BUSY_Pin_name_BIT       2
#define Touch_BUSY_Pin_Function       FUNC0
#define Touch_BUSY_GPIO_PORT          4     
#define Touch_BUSY_GPIO_BIT           9



#define LCD_43INCH_Touch_CS_Pin_name_PORT      0X0A
#define LCD_43INCH_Touch_CS_Pin_name_BIT       1
#define LCD_43INCH_Touch_CS_Pin_Function       FUNC0
#define LCD_43INCH_Touch_CS_GPIO_PORT          4     
#define LCD_43INCH_Touch_CS_GPIO_BIT           8

/******************************* 7 inch LCD TOUCHPANEL *****************************************/

#define LCD_7INCH_Touch_CS_Pin_name_PORT      0X0C
#define LCD_7INCH_Touch_CS_Pin_name_BIT       2
#define LCD_7INCH_Touch_CS_Pin_Function       FUNC4
#define LCD_7INCH_Touch_CS_GPIO_PORT          6     
#define LCD_7INCH_Touch_CS_GPIO_BIT           1


			


/*********************************SSPI1 DEFININS **************************************/

#define SSP1_MISO_Pin_name_PORT    0X0F
#define SSP1_MISO_Pin_name_BIT     6
#define SSP1_MISO_Pin_Function     FUNC2

#define SSP1_MOSI_Pin_name_PORT    1
#define SSP1_MOSI_Pin_name_BIT     4
#define SSP1_MOSI_Pin_Function     FUNC5

#define SSP1_SCK_Pin_name_PORT     0x0F
#define SSP1_SCK_Pin_name_BIT      4
#define SSP1_SCK_Pin_Function      FUNC0

#define SSP1_CS_Pin_name_PORT      0X0F
#define SSP1_CS_Pin_name_BIT       5
#define SSP1_CS_Pin_Function       FUNC4
#define SSP1_CS_GPIO_PORT          7     
#define SSP1_CS_GPIO_BIT           19

/*********************************NRF24L01 DEFININS **************************************/
//IRQ(P6_1)   CSN(P6_2)   CE(P6_4)

#define NRF24L01_IRQ_Pin_name_PORT      6
#define NRF24L01_IRQ_Pin_name_BIT       1
#define NRF24L01_IRQ_Pin_Function       FUNC0
#define NRF24L01_IRQ_GPIO_PORT          3     
#define NRF24L01_IRQ_GPIO_BIT           0

#define NRF24L01_CSN_Pin_name_PORT      6
#define NRF24L01_CSN_Pin_name_BIT       2
#define NRF24L01_CSN_Pin_Function       FUNC0
#define NRF24L01_CSN_GPIO_PORT          3     
#define NRF24L01_CSN_GPIO_BIT           1

#define NRF24L01_CE_Pin_name_PORT      6
#define NRF24L01_CE_Pin_name_BIT       4
#define NRF24L01_CE_Pin_Function       FUNC0
#define NRF24L01_CE_GPIO_PORT          3     
#define NRF24L01_CE_GPIO_BIT           3

/************************************************* **************************************/
#define Open_UART0
//#define Open_UART1
//#define Open_UART2
//#define Open_UART3
/*******************************UART0 DEFINITIONS***************************/
//UART0 
//RXD :  P2_1 ->FUNC1         
//TXD :  P2_0 ->FUNC1

#ifdef Open_UART0
#define USARTx                    (LPC_USARTn_Type *)LPC_USART0
#define USARTx_IRQn                USART0_IRQn     
#define UARTx_IRQHandler           UART0_IRQHandler
#endif

#ifdef Open_UART1
#define USARTx                    (LPC_USARTn_Type *)LPC_UART1
#define USARTx_IRQn                UART1_IRQn     
#define UARTx_IRQHandler           UART1_IRQHandler
#endif

#ifdef Open_UART2
#define USARTx                    (LPC_USARTn_Type *)LPC_USART2
#define USARTx_IRQn                USART2_IRQn     
#define UARTx_IRQHandler           UART2_IRQHandler
#endif

#ifdef Open_UART3
#define USARTx                    (LPC_USARTn_Type *)LPC_USART3
#define USARTx_IRQn                USART3_IRQn     
#define UARTx_IRQHandler           UART3_IRQHandler
#endif

#define U0_RXD_Pin_name_PORT       0X0F
#define U0_RXD_Pin_name_BIT        11
#define U0_RXD_pin_Function       FUNC1

#define U0_TXD_Pin_name_PORT       0X0F
#define U0_TXD_Pin_name_BIT        10
#define U0_TXD_pin_Function       FUNC1

/*******************************UART1 DEFINITIONS***************************/
//UART1 
//RXD :  PC_14 ->FUNC2        
//TXD :  PC_13 ->FUNC2

#define U1_RXD_Pin_name_PORT       0X0C
#define U1_RXD_Pin_name_BIT        14
#define U1_RXD_pin_Function       FUNC2

#define U1_TXD_Pin_name_PORT       0X0C
#define U1_TXD_Pin_name_BIT        13
#define U1_TXD_pin_Function       FUNC2


/*******************************UART2 DEFINITIONS***************************/
//UART2 
//RXD :  P2_11 ->FUNC2         
//TXD :  P2_10 ->FUNC2

#define U2_RXD_Pin_name_PORT       2
#define U2_RXD_Pin_name_BIT        11
#define U2_RXD_pin_Function       FUNC2

#define U2_TXD_Pin_name_PORT       2
#define U2_TXD_Pin_name_BIT        10
#define U2_TXD_pin_Function       FUNC2


/*******************************UART3 DEFINITIONS***************************/
//UART3
//RXD :  P9_4 ->FUNC7        
//TXD :  P9_3 ->FUNC7

#define U3_RXD_Pin_name_PORT       9
#define U3_RXD_Pin_name_BIT        4
#define U3_RXD_pin_Function       FU9NC7

#define U3_TXD_Pin_name_PORT       9
#define U3_TXD_Pin_name_BIT        3
#define U3_TXD_pin_Function       FUNC7

/*****************************DS18B20 DEFININS****************************************/

#define DS18B20_Pin_name_PORT      0X0E
#define DS18B20_Pin_name_BIT       14
#define DS18B20_GPIO_Function      FUNC4
#define DS18B20_GPIO_PORT          7    
#define DS18B20_GPIO_BIT           14





/************************************I2C1 DEFININS************************************************/

#define I2C1_SDA_Pin_name_PORT    2
#define I2C1_SDA_Pin_name_BIT     3
#define I2C1_SDA_Pin_Function     FUNC1

#define I2C1_SCL_Pin_name_PORT    2
#define I2C1_SCL_Pin_name_BIT     4
#define I2C1_SCL_Pin_Function     FUNC1

/************************************SDIO DEFININS************************************************/

#define SDIO_D0_Pin_name_PORT    1
#define SDIO_D0_Pin_name_BIT     9
#define SDIO_D0_Pin_Function     FUNC7

#define SDIO_D1_Pin_name_PORT    1
#define SDIO_D1_Pin_name_BIT     10
#define SDIO_D1_Pin_Function     FUNC7

#define SDIO_D2_Pin_name_PORT    1
#define SDIO_D2_Pin_name_BIT     11
#define SDIO_D2_Pin_Function     FUNC7

#define SDIO_D3_Pin_name_PORT    1
#define SDIO_D3_Pin_name_BIT     12
#define SDIO_D3_Pin_Function     FUNC7

#define SDIO_CMD_Pin_name_PORT    1
#define SDIO_CMD_Pin_name_BIT     6
#define SDIO_CMD_Pin_Function     FUNC7

#define SDIO_CD_Pin_name_PORT    1
#define SDIO_CD_Pin_name_BIT     13
#define SDIO_CD_Pin_Function     7

/************************************PS2 DEFININS************************************************/
#define PS2_DATA_Pin_name_PORT    6
#define PS2_DATA_Pin_name_BIT     4
#define PS2_DATA_pin_Function    FUNC0
#define PS2_DATA_GPIO_PORT        3     
#define PS2_DATA_GPIO_BIT         3  

#define PS2_CLK_Pin_name_PORT    6
#define PS2_CLK_Pin_name_BIT     5
#define PS2_CLK_pin_Function    FUNC0
#define PS2_CLK_GPIO_PORT        3     
#define PS2_CLK_GPIO_BIT         4


void LED_GPIO_Configuration(void);
void JOY_KEY_Configuration(void);


#endif
