#include "lpc4330_uart.h"




/*******************************************************************************
* Function Name  : LED_GPIO_Configuration
* Description    : None
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void UART_Configuration(void)
{
	
	// UART Configuration structure variable
	UART_CFG_Type UARTConfigStruct;
	// UART FIFO configuration Struct variable
	UART_FIFO_CFG_Type UARTFIFOConfigStruct;
	
#ifdef Open_UART0	
	scu_pinmux(U0_TXD_Pin_name_PORT ,U0_TXD_Pin_name_BIT , MD_PDN, U0_TXD_pin_Function); 	
	scu_pinmux(U0_RXD_Pin_name_PORT ,U0_RXD_Pin_name_BIT , MD_PLN|MD_EZI|MD_ZI, U0_RXD_pin_Function); 
#endif
	
#ifdef Open_UART1
	scu_pinmux(U1_TXD_Pin_name_PORT ,U1_TXD_Pin_name_BIT , MD_PDN, U1_TXD_pin_Function); 	
	scu_pinmux(U1_RXD_Pin_name_PORT ,U1_RXD_Pin_name_BIT , MD_PLN|MD_EZI|MD_ZI, U1_RXD_pin_Function); 	
#endif
	
#ifdef Open_UART2
	scu_pinmux(U2_TXD_Pin_name_PORT ,U2_TXD_Pin_name_BIT , MD_PDN, U2_TXD_pin_Function); 	
	scu_pinmux(U2_RXD_Pin_name_PORT ,U2_RXD_Pin_name_BIT , MD_PLN|MD_EZI|MD_ZI, U2_RXD_pin_Function); 	
#endif
	
	
	/* Initialize UART Configuration parameter structure to default state:
	 * Baudrate = 115200bps
	 * 8 data bit
	 * 1 Stop bit
	 * None parity
	 */
	UARTConfigStruct.Baud_rate= 115200;
	UARTConfigStruct.Databits= UART_DATABIT_8;
	UARTConfigStruct.Parity= UART_PARITY_NONE;
	UARTConfigStruct.Stopbits= UART_STOPBIT_1;
	UARTConfigStruct.Clock_Speed= 0;
	
		// Initialize UART0 peripheral with given to corresponding parameter
	UART_Init(USARTx, &UARTConfigStruct);

	/* Initialize FIFOConfigStruct to default state:
	 * 				- FIFO_DMAMode = DISABLE
	 * 				- FIFO_Level = UART_FIFO_TRGLEV0
	 * 				- FIFO_ResetRxBuf = ENABLE
	 * 				- FIFO_ResetTxBuf = ENABLE
	 * 				- FIFO_State = ENABLE
	 */	 
// 	UARTFIFOConfigStruct.FIFO_DMAMode = DISABLE;
// 	UARTFIFOConfigStruct.FIFO_Level = UART_FIFO_TRGLEV0;
// 	UARTFIFOConfigStruct.FIFO_ResetRxBuf = DISABLE;
// 	UARTFIFOConfigStruct.FIFO_ResetTxBuf = DISABLE;
	// Initialize FIFO for UART0 peripheral
	UART_FIFOConfig(USARTx, &UARTFIFOConfigStruct);
	
	// Enable UART Transmit
	UART_TxCmd(USARTx, ENABLE);	
	
}

/*******************************************************************************
* Function Name  : LED_GPIO_Configuration
* Description    : None
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void UART_NVIC_Config(void)
{
		    /* Enable UART Rx interrupt */
	UART_IntConfig(USARTx, UART_INTCFG_RBR, ENABLE);
	/* Enable UART line status interrupt */
	UART_IntConfig(USARTx, UART_INTCFG_RLS, ENABLE);
	
	/* preemption = 1, sub-priority = 1 */
    NVIC_SetPriority(USARTx_IRQn, ((0x01<<3)|0x01));
	/* Enable Interrupt for UART0 channel */
    NVIC_EnableIRQ(USARTx_IRQn);
}


/*----------------- INTERRUPT SERVICE ROUTINES --------------------------*/
/*********************************************************************//**
 * @brief		UART0 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void UARTx_IRQHandler(void)
{		 
	uint8_t Status;
	Status=UART_GetLineStatus(USARTx);
	printf("Status=0x%x\r\n",Status);
	printf("UART_ReceiveByte=%c\r\n",UART_ReceiveByte(USARTx));

}


/**
  * @brief  Retargets the C library printf function to the USARTx.
  * @param  None
  * @retval None
  */
int fputc(int ch, FILE *f)
{
	if(ch== '\n')
	{
		UART_SendByte(USARTx, '\r'); 
		while(UART_CheckBusy(USARTx));
   }
	 UART_SendByte(USARTx,(uint8_t) ch);
	 while(UART_CheckBusy(USARTx));
   return ch; 
		 
}



