/**********************************************************************
* $Id$		Gpio_LedBlinky.c	2011-06-02
*//**
* @file		Gpio_LedBlinky.c
* @brief	This example describes how to use GPIO to drive LEDs
* @version	1.0
* @date		02. June. 2011
* @author	NXP MCU SW Application Team
*
* Copyright(C) 2011, NXP Semiconductor
* All rights reserved.
*
***********************************************************************
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* products. This software is supplied "AS IS" without any warranties.
* NXP Semiconductors assumes no responsibility or liability for the
* use of the software, conveys no license or title under any patent,
* copyright, or mask work right to the product. NXP Semiconductors
* reserves the right to make changes in the software without
* notification. NXP Semiconductors also make no representation or
* warranty that such application will be suitable for the specified
* use without further testing or modification.
**********************************************************************/
#include "lpc43xx_gpio.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_libcfg_default.h"
#include "lpc43xx_cgu.h"
#include "LPC4357_IO.h"
#include "lpc4357_uart.h"
#include "UDA1380.h"
#include "i2c.h"
#include "string.h"
#include "waveDate.h"
#include "lpc43xx_i2s.h"


#define	  SNDTHELAST		(I2SSndPt  > WAVFILESIZ - 4)
/* 
 *  ļС 
 */
#define	  WAVFILESIZ		4800000
#define   MusicStartPos		45

uint32_t	Snd_End=0;			                /* Ϊ1ʱ,WAVļѾ*/
uint32_t 	I2SSndPt=0;			                /*  WAVָ   	 */


// /************************** PRIVATE VARIABLES ***********************/
 extern unsigned char audio[];

void I2S0_IRQHandler(void);
void  I2S_SndNchar( const unsigned char * datbuf, uint32_t* datbufpt_pt, uint32_t snd_N) ;


/* With ARM and GHS toolsets, the entry point is main() - this will
   allow the linker to generate wrapper code to setup stacks, allocate
   heap area, and initialize and copy code and data` segments. For GNU
   toolsets, the entry point is through __start() in the crt0_gnu.asm
   file, and that startup code will setup stacks and data */
int main(void)
{	
	volatile uint32_t LastDatToSnd_N;
  SystemInit();
	CGU_SetXTALOSC(12000000);
	CGU_EnableEntity(CGU_CLKSRC_XTAL_OSC, ENABLE);
	CGU_EntityConnect(CGU_CLKSRC_XTAL_OSC, CGU_CLKSRC_PLL1);
	CGU_SetPLL1(5);
	CGU_EnableEntity(CGU_CLKSRC_PLL1, ENABLE);
	CGU_EntityConnect(CGU_CLKSRC_PLL1, CGU_BASE_M4);
	CGU_UpdateClock();

	LED_GPIO_Configuration();
	UART_Configuration();
	printf("CGU_PERIPHERAL_I2S is clocked at %1.1f MHz\r\n", ((float)CGU_GetPCLKFrequency(CGU_PERIPHERAL_I2S)/1.0E6));	
	printf("CGU_PERIPHERAL_M4CORE is clocked at %1.1f MHz\r\n", ((float)CGU_GetPCLKFrequency(CGU_PERIPHERAL_M4CORE)/1.0E6));	
	printf("UDA1380 test procedures!!\r\n");
	
	I2SSndPt = MusicStartPos;
	UDA1380_I2S_Init();	
	printf("I2S INIT ok!!\r\n");
	UDA1380Init();	
	
	/* repeat the sound in a loop */
	
	while(1);	
}



/*----------------- INTERRUPT SERVICE ROUTINES --------------------------*/
/*********************************************************************//**
 * @brief		I2S IRQ Handler, call to send data to transmit buffer
 * @param[in]	None
 * @return 		None
 **********************************************************************/


void I2S0_IRQHandler()
{
		volatile uint32_t LastDatToSnd_N;
//		volatile uint32_t Snd_End 	;
		
	
		if (SNDTHELAST) 
		{					                            /* 跢͵Ѿ4*/
																																		 /* ʣµֽĿΪ123*/
		Snd_End = 1;																																/* ദ                 */	
		LastDatToSnd_N = WAVFILESIZ - I2SSndPt;
		I2S_SndNchar(WaveData, &I2SSndPt, LastDatToSnd_N); 
			
		} 
		else 
		{
			I2S_SndNchar(WaveData, &I2SSndPt, 4); 
			Snd_End = 0;	
		}
		
}


/*********************************************************************************************************
** Function name:       I2S_SndNchar
** Descriptions:        ݻdatbufĳʼָĿֽڵI2S_FIFO,Զݻķָ
** input parameters:    uint8*  datbuf:        跢ͳȥݻ׵ַ
**                      uint32* datbufpt_pt:   datbuf[*datbufpt_pt]ʼ4ֽ
**                      uint32  snd_N:         跢͵ݸ		
** output parameters:   
** Returned value:      
*********************************************************************************************************/
//void  I2S_SndNchar( uint8_t* datbuf, uint32_t* datbufpt_pt, uint32_t snd_N) 

void  I2S_SndNchar( const unsigned char * datbuf, uint32_t* datbufpt_pt, uint32_t snd_N) 
{
    uint32_t WavDat = 0;  	
    switch (snd_N)
	{ 		
    case 4:
       WavDat	= (datbuf[*datbufpt_pt  + 2]  << 24)
		| (datbuf[*datbufpt_pt + 3]   << 16)
		| (datbuf[*datbufpt_pt]       << 8)
		| (datbuf[*datbufpt_pt + 1]);
       break;
    case 3:
	WavDat 	= (datbuf[*datbufpt_pt + 2]   << 24)
		| (datbuf[*datbufpt_pt]       << 8)
		| (datbuf[*datbufpt_pt + 1]);
		
	break;
    case 2:
	WavDat  = (datbuf[*datbufpt_pt]       << 8)
		| (datbuf[*datbufpt_pt + 1]);
	break;
    case 1:
	WavDat  = datbuf[*datbufpt_pt];
	break;
    default:
        break;
    }
	
	I2S_Send(LPC_I2S0,WavDat);	
  *datbufpt_pt   += snd_N;	  				
}



#ifdef  DEBUG
/*******************************************************************************
* @brief		Reports the name of the source file and the source line number
* 				where the CHECK_PARAM error has occurred.
* @param[in]	file Pointer to the source file name
* @param[in]    line assert_param error line source number
* @return		None
*******************************************************************************/
void check_failed(uint8_t *file, uint32_t line)
{
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	/* Infinite loop */
	while(1);
}
#endif

/**
 * @}
 */
