#include "AT24C02.h"

/* Transmit setup */
I2C_M_SETUP_Type txsetup;
/* Receive setup */
I2C_M_SETUP_Type rxsetup;

#define I2Cx_SLVADDR		(0xA0>>1)

uint8_t wrdat[]=
{
		(0x01<<3),	// Memory Address, data will be written at this address
					// This internal memory address should be 8-byte alignment

		0x00,	// Data 0
		0x11,	// Data 1
		0x22,	// Data 2
		0x33,	// Data 3
		0x44,	// Data 4
		0x55,	// Data 5
		0x66,	// Data 6
		0x77,	// Data 7


};

uint8_t rddat[] = {
		// Don't use Memory address here, the Memory address to read
		// is the same as Memory Address to write.
		0x00,	// Data 0
		0x00,	// Data 1
		0x00,	// Data 2
		0x00,	// Data 3
		0x00,	// Data 4
		0x00,	// Data 5
		0x00,	// Data 6
		0x00,	// Data 7
};
void I2Cx_Init(uint32_t i2cClockFreq)
{

	/* I2C block ------------------------------------------------------------------- */
	// Initialize Slave I2C peripheral
	I2C_Init(LPC_I2C0, i2cClockFreq);
	/* Enable Slave I2C operation */
	I2C_Cmd(LPC_I2C0, ENABLE);
}


/*-------------------------PRIVATE FUNCTIONS------------------------------*/
/*********************************************************************//**
 * @brief		Write a number of data byte into EEPROM 24c0x
 * @param[in]	None
 * @return 		0: if success, otherwise (-1) returned.
 **********************************************************************/

int32_t I2Cx_Wirte(void)
{
	txsetup.sl_addr7bit = I2Cx_SLVADDR;
	txsetup.tx_data = wrdat;
	txsetup.tx_length = sizeof(wrdat);
//	txsetup.tx_length =2;
	txsetup.rx_data = NULL;
	txsetup.rx_length = 0;
	txsetup.retransmissions_max = 3;

	if (I2C_MasterTransferData(LPC_I2C0, &txsetup, I2C_TRANSFER_POLLING) == SUCCESS)
	{
		return (0);
	} else {
		return (-1);
	}
		
}
/*********************************************************************//**
 * @brief		Read a number of data byte from EEPROM 24c0x
 * @param[in]	None
 * @return 		0: if success, otherwise (-1) returned.
 **********************************************************************/
int32_t I2Cx_Read(void)
{

	rxsetup.sl_addr7bit = I2Cx_SLVADDR;
	rxsetup.tx_data = wrdat;	// Get address to read at writing address
	rxsetup.tx_length = 1;
	rxsetup.rx_data = rddat;
	rxsetup.rx_length = sizeof(rddat);
	rxsetup.retransmissions_max = 3;

	if (I2C_MasterTransferData(LPC_I2C0, &rxsetup, I2C_TRANSFER_POLLING) == SUCCESS){
		return (0);
	} else {
		return (-1);
	}
}	




