/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2011  SEGGER Microcontroller GmbH & Co. KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.12 - Graphical user interface for embedded applications **
All  Intellectual Property rights  in the Software belongs to  SEGGER.
emWin is protected by  international copyright laws.  Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with the following terms:

The software has been licensed to  ARM LIMITED whose registered office
is situated at  110 Fulbourn Road,  Cambridge CB1 9NJ,  England solely
for  the  purposes  of  creating  libraries  for  ARM7, ARM9, Cortex-M
series,  and   Cortex-R4   processor-based  devices,  sublicensed  and
distributed as part of the  MDK-ARM  Professional  under the terms and
conditions  of  the   End  User  License  supplied  with  the  MDK-ARM
Professional. 
Full source code is available at: www.segger.com

We appreciate your understanding and fairness.
----------------------------------------------------------------------
File        : main.c
Purpose     : Main program
---------------------------END-OF-HEADER------------------------------
*/

#include "GUI.h"
#include "lpc43xx_cgu.h"
#include "spifi_rom_api.h"

SPIFIobj sobj;

/* Default settings: */
/* Core @ 204MHz     */
/* SPIFI @ 102MHz    */
/* EMC @ 102MHz      */

//#define SPIFI80				/* Configure SPIFI @ 80MHz */
//#define CORE102				/* Configure core @ 102MHz */
//#define EMC51					/* Configure EMC at 51 MHz */

/*********************************************************************
*
*       Main
*/
int main (void) {
#if (USE_SPIFI)
  #ifdef SPIFI80
	  /* Configure SPIFI @ 80MHz */
	  spifi_init(&sobj, 9, S_RCVCLK | S_FULLCLK, 80);
	  CGU_SetDIV(CGU_CLKSRC_IDIVC, 2);
  #else
  	/* Configure SPIFI @ 102MHz */
	  spifi_init(&sobj, 9, S_RCVCLK | S_FULLCLK, 102);
	  CGU_EntityConnect(CGU_CLKSRC_IDIVD, CGU_BASE_SPIFI);
  #endif
#endif
#ifdef CORE102
	/* Configure core @ 102MHz */
	CGU_EntityConnect(CGU_CLKSRC_IDIVD, CGU_BASE_M4);
	#ifndef EMC51
		/* Configure EMC @ 102MHz */
	  LPC_CREG->CREG6 &= ~(1 << 16);       			/* EMC_CLK_DIV not divided by 2 */
		__ISB();
	  LPC_CCU1->CLK_M4_EMCDIV_CFG &= ~(1 << 5);	/* EMC_CLK_DIV not divided by 2 */
	  __ISB();
	#endif
#else
	#ifdef EMC51
	  #error "EMC clock cannot be 51MHz at 204MHz core clock!"
	#endif
#endif

	SystemInit();
//	CGU_Init();
	
	CGU_SetXTALOSC(12000000);
	CGU_EnableEntity(CGU_CLKSRC_XTAL_OSC, ENABLE);
	CGU_EntityConnect(CGU_CLKSRC_XTAL_OSC, CGU_CLKSRC_PLL1);
	// Disable PLL1 CPU hang???
	//CGU_EnableEntity(CGU_CLKSRC_PLL1, DISABLE);
	CGU_SetPLL1(15);
	CGU_EnableEntity(CGU_CLKSRC_PLL1, ENABLE);
	CGU_EntityConnect(CGU_CLKSRC_PLL1, CGU_BASE_M4);
	CGU_UpdateClock();

  MainTask();
  for (;;);
}

/*************************** End of file ****************************/
