/**********************************************************************
* $Id$		Gpio_LedBlinky.c	2011-06-02
*//**
* @file		Gpio_LedBlinky.c
* @brief	This example describes how to use GPIO to drive LEDs
* @version	1.0
* @date		02. June. 2011
* @author	NXP MCU SW Application Team
*
* Copyright(C) 2011, NXP Semiconductor
* All rights reserved.
*
***********************************************************************
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* products. This software is supplied "AS IS" without any warranties.
* NXP Semiconductors assumes no responsibility or liability for the
* use of the software, conveys no license or title under any patent,
* copyright, or mask work right to the product. NXP Semiconductors
* reserves the right to make changes in the software without
* notification. NXP Semiconductors also make no representation or
* warranty that such application will be suitable for the specified
* use without further testing or modification.
**********************************************************************/
#include "lpc43xx_gpio.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_libcfg_default.h"
#include "lpc43xx_cgu.h"
#include "LPC4357_IO.h"
#include "lpc4357_uart.h"
#include <includes.h>
#include "main.h"
#include "lpc43xx_utils.h"


/* Example group ----------------------------------------------------------- */
/** @defgroup Gpio_LedBlinky	Gpio_LedBlinky
 * @ingroup GPIO_Examples
 * @{
 */



/************************** PRIVATE VARIABLES *************************/
uint32_t msec;
static void taskStart (void  *parg);
static  void  taskLed1 (void  *parg);
static  void  taskLed2 (void  *parg);


/* With ARM and GHS toolsets, zthe entry point is main() - this will
   allow the linker to generate wrapper code to setup stacks, allocate
   heap area, and initialize and copy code and data segments. For GNU
   toolsets, the entry point is through __start() in the crt0_gnu.asm
   file, and that startup code will setup stacks and data */
int main(void)
{
//	uint32_t cclk;
  SystemInit();
	CGU_Init();
	
	UART_Configuration();
	UART_NVIC_Config();
	
	printf("CGU_PERIPHERAL_GPIO is clocked at %1.1f MHz\r\n", ((float)CGU_GetPCLKFrequency(CGU_PERIPHERAL_GPIO)/1.0E6));	
	printf("CGU_PERIPHERAL_M4CORE is clocked at %1.1f MHz\r\n", ((float)CGU_GetPCLKFrequency(CGU_PERIPHERAL_M4CORE)/1.0E6));	
	printf("CGU_PERIPHERAL_M4_BUS is clocked at %1.1f MHz\r\n", ((float)CGU_GetPCLKFrequency(CGU_PERIPHERAL_M4_BUS)/1.0E6));	
	printf("CGU_PERIPHERAL_SPIFI is clocked at %1.1f MHz\r\n", ((float)CGU_GetPCLKFrequency(CGU_PERIPHERAL_SPIFI)/1.0E6));	
	printf("CGU_PERIPHERAL_SSP1 is clocked at %1.1f MHz\r\n", ((float)CGU_GetPCLKFrequency(CGU_PERIPHERAL_SSP1)/1.0E6));	

	printf("Open4330 uCOSII-V2.91\r\n");
	
	// M3Frequency is automatically set when SetClock(BASE_M3_CLK... was called.
//	SysTick_Config(CGU_GetPCLKFrequency(CGU_PERIPHERAL_M4CORE)/1000);  				// Generate interrupt @ 1000 Hz


	OSInit();                                                                                          /*  Initialize the kernel of uC */
  OSTaskCreate ( taskStart,                                          
		           (void *)0, 
				   &GstkStart[TASK_START_STK_SIZE-1], 
				   TASK_START_PRIO );                                   /*  Initialize the start task   */
                                                                        /*  ʼ              */  
  OSStart(); /*  Start uC/OS-II uC/OS-II */

	return(0); 

}

/*********************************************************************************************************
** Function name:           taskStart	   
** Descriptions:            Start task	
** input parameters:        *parg
** output parameters:             
** Returned value:           
*********************************************************************************************************/
static void taskStart (void  *parg)
{
	  CPU_INT32U  cnts;
    (void)parg;
	
  //  cnts = CGU_GetPCLKFrequency(CGU_PERIPHERAL_M4CORE) / OS_TICKS_PER_SEC;
	// cnts = 1200;
	//	printf("cnts is clocked at %d\r\n", cnts);
		SysTick_Config(CGU_GetPCLKFrequency(CGU_PERIPHERAL_M4CORE)/1000);  				// Generate interrupt @ 1000 Hz
//  	OS_CPU_SysTickInit(cnts);
	
    #if OS_TASK_STAT_EN > 0
    OSStatInit();                                                       /*  Enable statistics           */                                                                       /*  ʹͳƹ                */
    #endif


  	/*
     *  Create the other tasks here. ﴴ
     */

		 OSTaskCreate (taskLed1, (void *)0,   		
								&GstkLed1[TASK_LED1_STK_SIZE-1], 
							TASK_LED1_PRIO);
	
		 OSTaskCreate (taskLed2, (void *)0,   		
								&GstkLed2[TASK_LED2_STK_SIZE-1], 
							TASK_LED2_PRIO); 	
		while (1)
		{                             
        OSTaskSuspend(OS_PRIO_SELF);                                    /*  The start task can be pended*/
                                                                       /*  here.   */
    }

}


static  void  taskLed1 (void  *parg)
{
    (void)parg;
		LED_GPIO_Configuration(); 
    while (1) 
    {
			GPIO_ClearValue(LED1_GPIO_PORT,(1<<LED1_GPIO_BIT));	             	
			printf("taskLed OFF\r\n");
			OSTimeDly(100);
	}
}


static  void  taskLed2 (void  *parg)
{
    (void)parg;
//		LED_GPIO_Configuration(); 
    while (1) 
    {	             	////////////////////////////////////////////
			printf("taskLed ON \r\n");
			GPIO_SetValue(LED1_GPIO_PORT,(1<<LED1_GPIO_BIT));
			OSTimeDly(200);
	}
}


#ifdef  DEBUG
/*******************************************************************************
* @brief		Reports the name of the source file and the source line number
* 				where the CHECK_PARAM error has occurred.
* @param[in]	file Pointer to the source file name
* @param[in]    line assert_param error line source number
* @return		None
*******************************************************************************/
void check_failed(uint8_t *file, uint32_t line)
{
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	/* Infinite loop */
	while(1);
}
#endif

/**
 * @}
 */
